/*
 * SimpleMail.cpp
 *
 * Written by Matt Pauker <mpauker@peganet.com>
 *        and Jon Watte   <hplus@ix.netcom.com>
 *
 * Last modified: 7/8/96
 */

#include <SupportDefs.h>
#include <sys/types.h>
#include <netdb.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

enum {
	false, true
};

static void
scan_send(
	FILE *file,
	char *text,
	int bufSize,
	int mySocket,
	bool escape = true)
{
	bool done;

	done = FALSE;
	bool last = true;

	while (!done)
	{
		memset(text, 0, bufSize);
		fgets(text, bufSize-1, file);	   // Get the line of text
		text[bufSize-1] = 0;
		if ((strcmp(text, ".\n") != 0) && !feof(file) && !ferror(file))
		{
			if (last && escape) { // did we get a newline on last line?
				if (text[0] == '~') {
					switch (text[1]) {
					case 'q':
						closesocket(mySocket);
						exit(1);	//      bye, bye!
					case 'r':
						{
							text[strlen(text)-1] = 0;
							char *ptr = &text[2];
							while (*ptr == ' ') ptr++;
							FILE *f = fopen(ptr, "r");
							if (f) {
								fprintf(stderr, "Reading file: %s\n", ptr);
								scan_send(f, text, bufSize, mySocket, false);
								fclose(f);
							} else {
								fprintf(stderr, "Can't open: %s\n", ptr);
							}
						}
						break;
					default:
						fprintf(stderr, "Usage: ~q (quit) or ~r (read)\n");
						break;
					}
				continue;
				}
			}

			// If the text starts with . or from, add a >
			if (strncmp(text, ".", 1) == 0) 
				send(mySocket, ">", 1, 0);
			else if ((strncmp(text, "From", 4) == 0) || (strncmp(text, "from", 4) ==0))
				send(mySocket, ">", 1, 0);
			if (!strrchr(text, '\n')) {
				strcat(text, "\n");
				last = false;
			} else {
				last = true;
			}
			
			send(mySocket, text, strlen(text), 0);	  // Send it
		}
		else
			done = TRUE;
	}
}



int main(int argc, char *argv[])
{
	char *host;
	struct hostent *hostInfo;
	struct sockaddr_in sa;
	int mySocket;
	long x;

	if (argc == 1)  // Make sure there are some addresses
	{
		printf("Syntax: mail <user>@<host>\n");
		exit(-1);
	}

	host = getenv("SMTPSERVER");    // Get the SMTP server's address
	if (host == NULL)
	{
		printf("You don't have an SMTP server specified.  Set the SMTPSERVER shell variable\n");
		printf("to the name of the server (env SMTPSERVER=<server>).\n");
		printf("You should also set your return address in RETURNTO.\n");
		exit(-1);
	}

	// Setup socket
	if ((hostInfo = gethostbyname(host)) == NULL)
	{
		printf("Unknown host.\n");
		exit(-1);
	}
	
	memset(&sa, 0, sizeof(sa));
	memcpy((char *)&sa.sin_addr, hostInfo->h_addr, hostInfo->h_length);

	// Set address
	sa.sin_family = hostInfo->h_addrtype;
	sa.sin_port = htons(25);
	
	// Create it
  if ((mySocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1)
	{
		printf("Couldn't open socket.\n");
		exit(-1);
	}
	
	// Try to connect to the host
	if (connect(mySocket, (struct sockaddr *)&sa, sizeof(sa)) < 0)
	{
		printf("Couldn't connect.\n");
		closesocket(mySocket);
		exit(-1);
	}

	// Whee...Credits
	printf("SimpleMail 1.1\n");
	printf("Written by Matt Pauker <mpauker@peganet.com>\n");
	printf("       and Jon Watte   <hplus@ix.netcom.com>\n");
	printf("\n");
	
	char *beboxUser = (char *)malloc(50);
	char *beboxHost = (char *)malloc(100);
	
	// Get the user and host names of the BeBox
	gethostname(beboxHost, 100);
	getusername(beboxUser, 50);
	
	// Create and send the HELO command
	char *helloMsg = (char *)malloc(100);
	
	sprintf(helloMsg, "HELO %s\n", beboxHost);
	send(mySocket, helloMsg, strlen(helloMsg), 0);

	free(helloMsg);

	// Create and send the MAIL FROM: command
	char *mailFromMsg = (char *)malloc(100);
	char *returnTo = getenv("RETURNTO");
	if (!returnTo) {
		fprintf(stderr, "No RETURNTO, using defaults\n");
		sprintf(mailFromMsg, "MAIL FROM: <%s@%s>\n", beboxUser, beboxHost);
	} else {
		sprintf(mailFromMsg, "MAIL FROM: %s\n", returnTo);
	}
	send(mySocket, mailFromMsg, strlen(mailFromMsg), 0);
	fprintf(stderr, mailFromMsg);

	free(mailFromMsg);

	// Create and send the RCPT TO: command(s)
	char *rcptToMsg = (char *)malloc(100);
	for (x = 1; x < argc; x++)
	{
		sprintf(rcptToMsg, "RCPT TO: <%s>\n", argv[x]);
		send(mySocket, rcptToMsg, strlen(rcptToMsg), 0);
	}
	free(rcptToMsg);

	// Send the DATA command
	send(mySocket, "DATA\n", 5, 0);

	// Create and send Subject: header
	char *header = (char *)malloc(1000);    
	char *subject = (char *)malloc(100);
	
	printf("Enter subject: ");
	fflush(stdout);
	gets(subject);
	if (!subject[0]) {
		fprintf(stderr, "No subject.\n");
		exit(1);
	}
	sprintf(header, "Subject: %s", subject);
	strcat(header, "\n");
	
	free(subject);
	
	//      Send Subject: header
	send(mySocket, header, strlen(header), 0);

	// Create and send From: header
	if (!returnTo) {
		sprintf(header, "From: <%s@%s>\n", beboxUser, beboxHost);
	} else {
		sprintf(header, "From: %s\n", returnTo);
	}
	send(mySocket, header, strlen(header), 0);

	// Create and send To: header
	strcpy(header, "To: ");
	for (x = 1; x < (argc - 1); x++)
	{
		strcat(header, argv[x]);
		strcat(header, ", ");
	}

	strcat(header, argv[argc - 1]);
	strcat(header, "\n");
	send(mySocket, header, strlen(header), 0);

	free(header);

  // Message text
	char *text = (char *)malloc(500);
	bool done;
	
	printf("Enter text, . to stop:\n");

	scan_send(stdin, text, 500, mySocket);

	free(text);

	send(mySocket, ".\n", 2, 0);	    // End of message command
	send(mySocket, "QUIT\n", 5, 0); // Let's get out of here...

	closesocket(mySocket);	  // Don't forget to clean up...

	printf("Message sent.\n");

	return 0;
}